# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: MIT-0
#
import json
import urllib.parse
import boto3
import datetime
import dateutil.parser as p
import random 
import string 
import botocore
import hashlib
import logging
logger = logging.getLogger()
logger.setLevel(logging.INFO)

s3 = boto3.client('s3')

def _list_s3_objects(bucket,prefix,continuation_key=None):

    try:
        if continuation_key is not None:
            response=s3.list_objects_v2(Bucket=bucket,Prefix=prefix,Delimiter = "/",ContinuationToken=continuation_key)
            logger.info(response)
        else:
            response=s3.list_objects_v2(Bucket=bucket,Prefix=prefix,Delimiter = "/")
        return {"statusCode":response['ResponseMetadata']['HTTPStatusCode'],"body":response}
    except botocore.exceptions.ClientError as e:
        logger.exception(error.response)
        raise(e)


def _check_date_difference(processing_time, last_modified,last_successful_run_time):
    
    processing_time=p.isoparse(processing_time)

    if last_successful_run_time=="FIRST_RUN":
        return processing_time>last_modified
    else:
        last_successful_run_time=p.isoparse(last_successful_run_time)
        return last_successful_run_time<last_modified<processing_time

def _process_s3_response(bucket,response,object_list,processing_time_string,dest_bucket_name,dest_prefix,folder_hash,last_successful_run_time):
    for key in response:
        if _check_date_difference(processing_time=processing_time_string, last_modified=key['LastModified'],last_successful_run_time=last_successful_run_time) is True:
            object_name=_extract_object_name(key["Key"])
            new_object_name=_create_new_object_name(original_object_name=object_name,new_prefix=dest_prefix)
            json_obj={"folderHash":folder_hash,"srcBucket": bucket, "copySource": key["Key"], "destBucketName": dest_bucket_name, "destPrefixAndKey":new_object_name}
            object_list.append(json_obj)
    return object_list

def _extract_object_name(copy_source):
    return copy_source.rsplit('/',1)[1]

def _extract_source_bucket_name(copy_source,split=1,index=0):
    return copy_source.split('/',split)[index]

def _create_new_object_name(original_object_name,new_prefix):
    random_string = ''.join(random.choices(string.ascii_uppercase + string.digits, k=15))

    return new_prefix+random_string+"_"+original_object_name

def _create_folder_hash(dest_prefix,execute_time):
    folder_hash=hashlib.md5(execute_time.encode('utf-8')).hexdigest()
    return dest_prefix+str(folder_hash)+"/",folder_hash
    
    
def _process_response(response,tenant_id,dest_prefix,processing_time_str,src_bucket,dest_bucket_name,last_successful_run):
    if (response['statusCode']!=200):
        return {"statusCode":response['statusCode'],"Content": {"tenantId":tenant_id, "status": "Error"}}
    elif (response['statusCode']==200 and 'Contents' not in response['body']):
        return {"statusCode":response['statusCode'],"Content": {"tenantId":tenant_id,"status":"NothingToProcess"}}

    else:
        #create hash for the folder name that will be used as destination
        dest_prefix_with_folder_hash,folder_hash=_create_folder_hash(dest_prefix,processing_time_str)
        object_list = []
        object_list=_process_s3_response(bucket=src_bucket,response=response['body']['Contents'],object_list=object_list,processing_time_string=processing_time_str,dest_bucket_name=dest_bucket_name,dest_prefix=dest_prefix_with_folder_hash,folder_hash=folder_hash,last_successful_run_time=last_successful_run)
            
        while response['body']['IsTruncated']:
            continuation_key = response['body']['NextContinuationToken']
            response = _list_s3_objects(bucket=src_bucket,prefix=src_prefix,continuation_key=continuation_key)
            object_list=_process_s3_response(bucket=src_bucket,response=response['body']['Contents'],object_list=object_list,processing_time_string=processing_time_str,dest_bucket_name=dest_bucket_name,dest_prefix=dest_prefix_with_folder_hash,folder_hash=folder_hash,last_successful_run_time=last_successful_run)
    
        if(len(object_list)==0):
            return {"statusCode":200,"Content":{"status": "NothingToProcess","tenantId":tenant_id}}
        else:
            return {"statusCode":200,"Content":{"status": "ItemsToProcess", "objectList":object_list}}

def lambda_handler(event, context):
    logger.info("Received event: " + json.dumps(event, indent=2))

    # Get the object from the event and show its content type
    data_in=event['input']
    src_bucket = data_in['s3']['config']['srcBucketName']
    src_prefix = data_in['s3']['config']['srcPrefix']
    tenant_id = data_in['tenantId']
    database_name = data_in['sourceConfig']['databaseName']
    table_name = data_in['sourceConfig']['tableName'] 
    processing_time_str = data_in['executionTime']['time']
    
    dest_prefix=data_in['s3']['config']['destPrefix']
    dest_bucket_name=data_in['s3']['config']['destBucketName']
    #src_prefix ends with "/"
    full_src_prefix= src_prefix+tenant_id+"/"+database_name+"/"+table_name+"/"

    last_successful_run =data_in['lastSuccessfulRun']['Content']
    response=_list_s3_objects(bucket=src_bucket,prefix=full_src_prefix)
    processed_response=_process_response(response,tenant_id,dest_prefix,processing_time_str,src_bucket,dest_bucket_name,last_successful_run)
    return processed_response

